/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import { Card } from "semantic-ui-react";
import { GenericIcon } from "../icon";
import React from "react";
import classNames from "classnames";
/**
 * Selection card component.
 *
 * @param {SelectionCardPropsInterface} props - Props injected to the components.
 *
 * @return {React.ReactElement}
 */
export const SelectionCard = (props) => {
    const { className, description, disabled, header, id, inline, image, imageSize, onClick, selected, size, spaced, textAlign } = props;
    const classes = classNames("selection-card", {
        disabled,
        inline,
        selected,
        [size]: size,
        [`spaced-${spaced}`]: spaced,
        ["with-image"]: image
    }, className);
    return (React.createElement(Card, { id: id, className: classes, onClick: onClick, link: false, as: "div" },
        image && (React.createElement(Card.Content, { className: "card-image-container" },
            React.createElement(GenericIcon, { className: "card-image", size: imageSize, icon: image, square: true, transparent: true }))),
        React.createElement(Card.Content, { className: "card-text-container", style: { textAlign } },
            React.createElement(Card.Header, null, header),
            description && React.createElement(Card.Description, null, description))));
};
SelectionCard.defaultProps = {
    imageSize: "tiny",
    inline: false,
    onClick: () => null,
    size: "default",
    textAlign: "center"
};
//# sourceMappingURL=selection-card.js.map